import type { JsonValue, Jsonify } from 'type-fest'

declare type RuntimeContext = 'devtools' | 'background' | 'popup' | 'options' | 'content-script' | 'window'
interface Endpoint {
  context: RuntimeContext
  tabId: number
  frameId?: number
}
interface IBridgeMessage<T extends JsonValue> {
  sender: Endpoint
  id: string
  data: T
  timestamp: number
}
declare type OnMessageCallback<T extends JsonValue, R = void | JsonValue> = (message: IBridgeMessage<T>) => R | Promise<R>
interface IInternalMessage {
  origin: Endpoint
  destination: Endpoint
  transactionId: string
  hops: string[]
  messageID: string
  messageType: 'message' | 'reply'
  err?: JsonValue
  data?: JsonValue | void
  timestamp: number
}
interface StreamInfo {
  streamId: string
  channel: string
  endpoint: Endpoint
}
interface HybridUnsubscriber {
  (): void
  dispose: () => void
  close: () => void
}
declare type Destination = Endpoint | RuntimeContext | `${RuntimeContext}@${number}`
declare const ProtocolWithReturnSymbol: unique symbol
interface ProtocolWithReturn<Data, Return> {
  data: Jsonify<Data>
  return: Jsonify<Return>
  /**
     * Type differentiator only.
     */
  [ProtocolWithReturnSymbol]: true
}
/**
 * Extendable by user.
 */
interface ProtocolMap {
}
declare type DataTypeKey = keyof ProtocolMap extends never ? string : keyof ProtocolMap
declare type GetDataType<K extends DataTypeKey, Fallback extends JsonValue = undefined> = K extends keyof ProtocolMap ? ProtocolMap[K] extends ProtocolWithReturn<infer Data, any> ? Data : ProtocolMap[K] : Fallback
declare type GetReturnType<K extends DataTypeKey, Fallback extends JsonValue = undefined> = K extends keyof ProtocolMap ? ProtocolMap[K] extends ProtocolWithReturn<any, infer Return> ? Return : void : Fallback

export { Destination as D, Endpoint as E, GetDataType as G, HybridUnsubscriber as H, IBridgeMessage as I, OnMessageCallback as O, ProtocolWithReturn as P, RuntimeContext as R, StreamInfo as S, IInternalMessage as a, ProtocolMap as b, DataTypeKey as c, GetReturnType as d }
